// -*- C++ -*-
/*!
 * @file Properties.h
 * @brief Property list class (derived from Java Properties)
 * @date $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: Properties.h 1971 2010-06-03 08:46:40Z n-ando $
 *
 */

#ifndef COIL_PROPERTIES_H
#define COIL_PROPERTIES_H


#include <string>
#include <vector>
#include <map>
#include <climits>


/*!
 * @if jp
 * @namespace coil 
 *
 * @brief Common Object Interface Layer
 *
 *
 * @else
 * @namespace coil
 *
 * @brief Common Object Interface Layer
 *
 *
 * @endif
 */
namespace coil
{
  /*!
   * @if jp
   *
   * @class Properties
   * @brief ץѥƥåȤɽ륯饹
   *
   * Properties 饹ϡѤΥץѥƥåȤɽ Properties 򥹥ȥ꡼
   * ݴɤꡢȥ꡼फɤꤹ뤳ȤǤ롣
   * ץѥƥꥹȤγƥӤбͤʸȤʤäƤ롣
   *
   * ץѥƥꥹȤˤϡΡ֥ǥե͡פȤ̤ΥץѥƥꥹȤ
   * ȤǤ롣ΥץѥƥꥹȤǥץѥƥĤʤȡ
   * 2ܤΥץѥƥꥹȤ롣 
   *
   * ץѥƥμˤ getProperty() ץѥƥΥåȤˤ setProperty() 
   * ä᥽åɤѤ뤳Ȥ侩롣
   *
   * ץѥƥ򥹥ȥ꡼¸Ȥޤϥȥ꡼फɤȤ
   * ˡISO 8859-1 ʸ󥳡ǥ󥰤Ѥ롣Υ󥳡ǥ󥰤
   * ľɽǤʤʸϡȤǤʤ
   *
   * Υ饹ϡJava  Properties 饹 (java.util.Properties) ȤۤƱͤ
   * ᥽åɤġޤϤե Java  Properties 饹
   * ϤΤȸߴ뤬Unicode ޤΤϰȤǤʤ
   *
   * @since 0.4.0
   *
   * @else
   *
   * @class Properties
   * @brief Class represents a set of properties
   *
   * The Properties class represents a persistent set of properties. The
   * Properties can be saved to a stream or loaded from a stream. Each key and
   * its corresponding value in the property list is a string. 
   *
   * A property list can contain another property list as its "defaults". This
   * second property list is searched if the property key is not found in the
   * original property list. 
   *
   * It is recommended to use these method; setProperty() to get properties,
   * setProperty() to set properties. 
   *
   * When propertis are stored in a stream or when they are loaded from
   * the stream, the ISO 8859-1 character encoding is used.
   * Characters that cannot be directly represented in this encoding can be used.
   *
   * This class has almost same methods of Java's Properties class
   * (java.util.Properties). Also, input and output files are compatible with 
   * outputs of Java's Properties class, but Unicode encoded property file
   * is not be supported.
   *
   * @endif
   */
  class Properties
  {
  public:
    /*!
     * @if jp
     *
     * @brief 󥹥ȥ饯(rootΡɤΤߺ)
     *
     * key  value ΤߤͿ Property Υ롼ȥΡɤ롣
     * ͤƥǥեͤȤꤵ롣
     *
     * @param key ץѥƥΥ(ǥե:"")
     * @param value ץѥƥ(ǥե:"")
     * 
     * @else
     *
     * @brief Constructor(Create only root node)
     *
     * Create a root node of Property with root's key and value.
     * All values are set as default value.
     *
     * @param key Properties's keys(The default values:"")
     * @param value Properties's values(The default values:"")
     * 
     * @endif
     */
    Properties(const char* key = "", const char* value = "");
    
    /*!
     * @if jp
     *
     * @brief 󥹥ȥ饯(mapǥǥեͤͿ)
     *
     * std::string  std::map ǥեͤˤ Properties 롣
     * ͤƥǥեͤȤꤵ롣
     * 
     * @param defaults ǥեͤȤƻꤵmap
     * 
     * @else
     *
     * @brief Constructor(Give the default value with map)
     *
     * Create Properties with default value of std::string map.
     * All values are set as default value.
     * 
     * @param defaults map that is spcified as the default value
     * 
     * @endif
     */
    Properties(std::map<std::string, std::string>& defaults);
    
    /*!
     * @if jp
     *
     * @brief 󥹥ȥ饯(char*[] ǥǥեͤͿ)
     *
     * ꤵ줿ǥեͤĶΥץѥƥꥹȤ롣
     * ͤƥǥեͤȤꤵ롣
     * ǥեͤ char* ˤͿ졢key  value Фˤʤä
     * ꡢꥹȤνüοɽ num ʸ key Ϳ
     * ʤФʤʤ
     * ʲ򼨤
     *
     * <pre>
     * const char* defaults = {
     *     "key1", "value1",
     *     "key2", "value2",
     *     "key3", "value3",
     *     "key4", "value4",
     *     "key5", "value5",
     *     "" };
     * Properties p(defaults);
     * // ⤷
     * Properties p(defaults, 10);
     * </pre>
     * 
     * @param defaults ǥեͤꤹ
     * @param num ǥեͤꤹǿ(ǥե:LONG_MAX)
     * 
     * @else
     *
     * @brief Constructor(Give the default value with char*[])
     *
     * Creates an empty property list with the specified defaults.
     * All values are set as the default values.
     * The default values are given by array of char*, which should be pairs
     * of "key" and "value". The end of list is specified by argument "num"
     * which specifies number of array, or null character of key.
     * The following is an example.
     *
     * <pre>
     * const char* defaults = {
     *     "key1", "value1",
     *     "key2", "value2",
     *     "key3", "value3",
     *     "key4", "value4",
     *     "key5", "value5",
     *     "" };
     * Properties p(defaults);
     * // or
     * Properties p(defaults, 10);
     * </pre>
     * 
     * @param defaults Array that specifies the default values
     * @param num Number of elements that specifies the default value
     *            (The default value:LONG_MAX)
     * 
     * @endif
     */
    Properties(const char* defaults[], long num = LONG_MAX);
    
    /*!
     * @if jp
     * @brief ԡ󥹥ȥ饯
     *
     * Ϳ줿 Properties Υͤӥǥեͤ
     * ƤΤޤޥԡ롣
     *
     * @else
     *
     * @brief Copy Constructor
     *
     * All of given Properties's keys, values and default values 
     * are copied to new Properties.
     * 
     * @endif
     */
    Properties(const Properties& prop);
    
    /*!
     * @if jp
     * @brief 黻
     *
     * ͤ Properties Υͤӥǥեͤƺ졢
     * ͤ Properties ΥͤӥǥեͤƤΤޤ
     * ԡ롣
     *
     * @else
     * @brief Assignment operator
     *
     * All Properties's keys, values and default values of left side
     * are deleted, all Properties's keys, values and default values of
     * right side are copied to new Properties.
     *
     * @endif
     */
    Properties& operator=(const Properties& prop);
    
    /*!
     * @if jp
     *
     * @brief ǥȥ饯
     *
     * @else
     *
     * @brief Destructor
     *
     * @endif
     */
    virtual ~Properties(void);
    
    //============================================================
    // public functions
    //============================================================
    
    /*!
     * @if jp
     * @brief Name μ
     *
     * ץѥƥ̾Τ롣
     *
     * @return ץѥƥ̾
     *
     * @else
     * @brief Get Names
     *
     * Get Properties's names.
     *
     * @return Properties's names
     *
     * @endif
     */
    inline const char* getName(void) const          {return name.c_str();}
    
    /*!
     * @if jp
     * @brief ͤμ
     *
     * ץѥƥͤ롣
     *
     * @return ץѥƥ
     *
     * @else
     * @brief Get values
     *
     * Get Properties's values.
     *
     * @return Properties's values
     *
     * @endif
     */
    inline const char* getValue(void) const         {return value.c_str();}
    
    /*!
     * @if jp
     * @brief ǥեͤμ
     *
     * ץѥƥΥǥեͤ롣
     *
     * @return ץѥƥǥե
     *
     * @else
     * @brief Get default values
     *
     * Get Properties's default values.
     *
     * @return Properties's default values
     *
     * @endif
     */
    inline const char* getDefaultValue(void) const {return default_value.c_str();}
    
    /*!
     * @if jp
     * @brief Ǥμ
     *
     * ץѥƥλǤ롣
     *
     * @return 
     *
     * @else
     * @brief Get elements of leaf
     *
     * Get Properties's elements of leaf.
     *
     * @return Elements of leaf
     *
     * @endif
     */
    inline const std::vector<Properties*>& getLeaf(void) const {return leaf;}
    
    /*!
     * @if jp
     * @brief 롼Ǥμ
     *
     * ץѥƥΥ롼Ǥ롣
     *
     * @return 롼
     *
     * @else
     * @brief Get root element
     *
     * Get Properties's root element.
     *
     * @return Root element
     *
     * @endif
     */
    inline const Properties* getRoot(void) const    {return root;}
    
    /*!
     * @if jp
     *
     * @brief ꤵ줿ĥץѥƥ򡢥ץѥƥꥹȤõ
     *
     * ꤵ줿ĥץѥƥ򡢥ץѥƥꥹȤõ
     * ΥץѥƥꥹȤˤʤȡǥեȤΥץѥƥꥹȡ
     * ˤΥǥե֤ͤĴ٤롣
     * ΥץѥƥĤʤϡnull ֤롣 
     *
     * @param key ץѥƥ
     *
     * @return ꤵ줿ͤĤΥץѥƥꥹȤ
     *
     * @else
     *
     * @brief Search for the property with the specified key in this property
     *
     * Search for the property with the specified key in this property list.
     * If the key is not found in this property list, the default property list,
     * and its defaults, recursively, are then checked. The method returns null
     * if the property is not found. 
     *
     * @param key The property key.
     *
     * @return The value in this property list with the specified key value.
     *
     * @endif
     */
    const std::string& getProperty(const std::string& key) const;
    
    /*!
     * @if jp
     *
     * @brief ꤵ줿ĥץѥƥ򡢥ץѥƥꥹȤõ
     *
     * ꤵ줿ĥץѥƥ򡢥ץѥƥꥹȤõ
     * ΥץѥƥꥹȤˤʤϡǥեͤΰ֤롣 
     *
     * @param key ץѥƥ
     * @param def ǥե
     *
     * @return ꤵ줿ͤĤΥץѥƥꥹȤ
     *
     * @else
     *
     * @brief Search for the property with the specified key in property list
     *
     * Search for the property with the specified key in this property list.
     * The method returns the default value argument if the property is not 
     * found.
     *
     * @param key The property key
     * @param def The  default value. 
     *
     * @return The value in this property list with the specified key value.
     *
     * @endif
     */
    const std::string& getProperty(const std::string& key,
				   const std::string& def) const;
    
    /*!
     * @if jp
     *
     * @brief ꤵ줿ĥץѥƥ򡢥ץѥƥꥹȤõ
     *
     * ꤵ줿ĥץѥƥ֤
     * ΥץѥƥꥹȤˤʤХǥե֤ͤ
     * ˸ĤʤСʸ֤
     *
     * @param key ץѥƥ
     *
     * @return ꤵ줿ͤĤΥץѥƥꥹȤ
     *
     * @else
     *
     * @brief Search for the property with the specified key in property list
     *
     * Search for the property with the specified key in this property list.
     * If the key is not found in this property list, the default property list,
     * and its defaults, recursively, are then checked. The method returns 
     * empty string if the property is not found. 
     *
     * @param key The property key
     *
     * @return The value in this property list with the specified key value.
     *
     * @endif
     */
    const std::string& operator[](const std::string& key) const;
    
    /*!
     * @if jp
     *
     * @brief ꤵ줿ĥץѥƥ򡢥ץѥƥꥹȤõ
     *
     * ꤵ줿ĥץѥƥ֤
     * ΥͤץѥƥꥹȤˤʤХǥե֤ͤ
     * ˸ĤʤСʸ֤
     * ͤˤʤˡƱͤǤʤȤͿ줿
     * бץѥƥ˱ͤ
     *
     * @param key ץѥƥ
     *
     * @return ꤵ줿ͤĤΥץѥƥꥹȤ
     *
     * @else
     *
     * @brief Search for the property with the specified key in property list
     *
     * Search for the property with the specified key in this property list.
     * If the key is not found in this property list, the default property list,
     * and its defaults, recursively, are then checked. The method returns 
     * empty string if the property is not found.
     * If there is no element with the same value in the left value, insert the
     * right value in corresponding property.
     *
     * @param key The property key
     *
     * @return The value in this property list with the specified key value.
     *
     * @endif
     */
    std::string& operator[](const std::string& key);
    
    /*!
     * @if jp
     * @brief ꤵ줿Фƥǥեͤ
     *
     * ꤵ줿ĥץѥƥΥǥե֤ͤ
     * ꤵ줿ĥץѥƥ¸ߤʤˤ϶ʸ֤
     *
     * @param key ץѥƥ
     *
     * @return ꤵ줿ͤĥץѥƥΥǥե
     *
     * @else
     * @brief Get the default values with specified key.
     *
     * Return the default values with specified key.
     * If the property with specified key does not exist, the method returns
     * empty string.
     *
     * @param key The property key
     *
     * @return The value in this property list with the specified key value.
     *
     * @endif
     */
    const std::string& getDefault(const std::string& key) const;
    
    /*!
     * @if jp
     *
     * @brief Properties  value  key ˤĤϿ
     *
     * Properties  value  key ˤĤϿ롣
     * Ǥ key ФͤäƤ硢ͤ˸Ť֤ͤ
     *
     * @param key ץѥƥꥹȤ֤륭
     * @param value key б 
     *
     * @return ץѥƥꥹȤλꤵ줿͡줬ʤ null
     *
     * @else
     *
     * @brief Set a value associated with key in the property list
     *
     * This method sets the "value" associated with "key" in the property list.
     * If the property list has a value of "key", old value is returned.
     *
     * @param key The key to be placed into this property list.
     * @param value The value corresponding to key. 
     *
     * @return The previous value of the specified key in this property list,
     *         or null if it did not have one.
     *
     *@endif
     */
    std::string setProperty(const std::string& key, const std::string& value);
    
    /*!
     * @if jp
     * @brief ǥեͤϿ
     *
     * key ǻꤵǤ˥ǥեͤϿ롣
     *
     * @param key ǥեͤϿץѥƥΥ
     * @param value Ͽǥե
     *
     * @return ꤵ줿ǥե
     *
     * @else
     * @brief Set a default value associated with key in the property list
     *
     * Set the default value to element specified by "key".
     *
     * @param key Property's key to set the default value
     * @param value The default value that is set
     *
     * @return The specified default value
     *
     * @endif
     */
    std::string setDefault(const std::string& key, const std::string& value);
    
    /*!
     * @if jp
     * @brief Properties ˥ǥեͤޤȤϿ
     *
     * ǻꤵ줿Ǥ˥ǥեͤޤȤϿ롣
     * ǥեͤ char* ˤͿ졢key  value Фˤʤä
     * ꡢꥹȤνüοɽ num ʸ key Ϳ
     * ʤФʤʤ
     * 
     * @param defaults ǥեͤꤹ
     * @param num ǥեͤꤹǿ(ǥե:LONG_MAX)
     * 
     * @else
     * @brief Set a default value together in the property list
     *
     * Set the default value to the element specified by array together in the
     * property list.
     * The default values are given by array of char*, which should be pairs
     * of "key" and "value". The end of list is specified by argument "num",
     * which specifies number of array or null character of key.
     * 
     * @param defaults Array that specifies the default values
     * @param num Number of elements that specifies the default value
     *            (Default value:LONG_MAX)
     * 
     * @endif
     */
    void setDefaults(const char* defaults[], long num = LONG_MAX);
    
    //============================================================
    // load and save functions
    //============================================================
    /*!
     * @if jp
     *
     * @brief ꤵ줿ϥȥ꡼ˡץѥƥꥹȤϤ
     *
     * ꤵ줿ϥȥ꡼ˡץѥƥꥹȤϤ롣
     * Υ᥽åɤϼ˥ǥХåѤ롣
     *
     * @param out ϥȥ꡼
     *
     * @else
     *
     * @brief Prints this property list out to the specified output stream
     *
     * Prints this property list out to the specified output stream.
     * This method is useful for debugging.
     *
     * @param out Output stream.
     *
     * @endif
     */
    void list(std::ostream& out);
    
    /*!
     * @if jp
     *
     * @brief ϥȥ꡼फ饭ǤФˤʤäץѥƥꥹȤɤ߹
     *
     * ϥȥ꡼फ饭ǤФˤʤäץѥƥꥹȤɤ߹ࡣ
     * ȥ꡼ϡISO 8859-1 ʸ󥳡ǥ󥰤ѤƤȤߤʤ롣
     * ƥץѥƥϡϥȥ꡼˹ñ̤ϿƤΤȤߤʤ졢
     * ƹԤϹԶڤʸ (\\n\\rޤ \\r\\n) ǽ롣
     * ϥȥ꡼फɤ߹Ԥϡϥȥ꡼ǥեν
     * ãޤǽ롣
     *
     * ʸιԡޤϺǽʸ ASCII ʸ # ޤ ! Ǥ
     * Ԥ̵뤵롣Ĥޤꡢ# ޤ ! ϥȹԤ򼨤
     *
     * Ԥޤϥȹ԰ʳΤ٤ƤιԤϡơ֥ɲäץѥƥ
     * 򵭽Ҥ롣Ԥν꤬ \ ξϡιԤз³ԤȤ
     *  (򻲾) ϡǽʸ顢ǽ ASCII ʸ
     * =:ޤ϶ʸľޤǤΡΤ٤Ƥʸ鹽롣
     *
     * ν򼨤ʸϡ \ դ뤳Ȥˤꥭ˴ޤ뤳Ȥ
     * Ǥ롣θζϤ٤ƥåפ롣
     * θκǽʸ = ޤ : ǤϡΥ
     * ̵뤵졢ΤȤζʸ⤹٤ƥåפ롣
     * ΤʳʸϤ٤ơϢʸΰȤʤ롣
     * ʸǤϡASCII ץ \\t\\n\\r\\\\\\"
     * \\'\\ (ߵȥڡ) \\uxxxx ǧ졢ñȤʸѴ
     * 롣
     * ޤԤκǸʸ \ ǤϡιԤϸߤιԤη³Ȥ
     * 롣ξ硢\ ȹԶڤʸ˴졢³ԤƬ˶
     * Ф⤹٤˴졢ʸΰˤϤʤʤ 
     *
     * ȤС 3 ԤϤ줾쥭 Truth ȴϢ Beauty ɽ
     * 
     * Truth = Beauty <BR>
     * Truth:Beauty <BR>
     * Truth\\t\\t\\t:Beauty <BR>
     *
     * ޤ 3 Ԥ 1 ĤΥץѥƥɽ 
     *
     * fruits\\t\\t\\t\\tapple, banana, pear, \ <BR>
     *                                  cantaloupe, watermelon, \ <BR>
     *                                  kiwi, mango <BR>
     *  fruits ǡǤ˴Ϣդ롣 
     * "apple, banana, pear, cantaloupe, watermelon, kiwi, mango"
     * ǽŪʷ̤ǥޤΤȤɬڡɽ褦ˡ
     *  \ ˥ڡ롣Ԥν򼨤 \ ȡ³ԤƬˤ
     * ˴졢¾ʸִʤ 
     * ޤ 3 ܤǤϡ cheeses ǡϢǤʸ
     * Ǥ뤳Ȥɽ 
     *
     * cheeses <BR>
     * ϡcheeses ǡϢǤ϶ʸǤ뤳ȤꤷƤ롣 
     *
     * @param inStream ϥȥ꡼ 
     *
     * @else
     *
     * @brief Loads property list that consists of key:value from input stream
     *
     * Reads a property list (key and element pairs) from the input stream.
     * The stream is assumed to be using the ISO 8859-1 character encoding.
     * Each property is assumed to be registered in the input stream by each
     * line, and each line terminator is should be a line break characters
     * (\\n or \\r or \\r\\n).
     * Lines are read from the input stream until end of file is reached. 
     *
     * A line that contains only white space characters or a line that its
     * first non-white space character is an ASCII '#' or '!' is ignored.
     * In a word, '#' or '!' represents comment lines.
     *
     * All lines except the blank line or comment line is described the property
     * that added to the table. However, if the line terminator is '\' and the 
     * next line continues, it is treated as a continuation line (See below).
     * The key is composed of all characters.
     * All of these key termination characters in the line starting with the 
     * first non-white space character and up to, but not including, the first
     * unescaped '=', ':', or white space character other than a line 
     * terminator. 
     * 
     * Line terminator characters can be included using \ escape sequences.
     * Any white space after the key is skipped.
     * If the first non-white space character after the key is '=' or ':',
     * then it is ignored and any white space characters after it are also 
     * skipped.
     * All remaining characters on the line become part of the associated element
     * string.
     * In element string, ASCII escape sequence such as \\t and \\n and \\r
     * and \\\\ and \\" and \\' and \\ (backslash character and space) 
     * and \\uxxxx have affect and they will be converted into a single 
     * character.
     * Also, if termination character in the line is \, the next line will be  
     * treated as continuing. In that case, \ and break character will be 
     * destroyed, and also its first space character will be destroyed, 
     * so these characters on the line will not become part of the element 
     * string.
     *
     * As an example, each of the following three lines specifies the key 
     * "Truth" and the associated element value "Beauty": 
     * 
     * Truth = Beauty <BR>
     * Truth:Beauty <BR>
     * Truth\\t\\t\\t:Beauty <BR>
     *
     * As another example, the following three lines specify a single 
     * property: 
     *
     * fruits\\t\\t\\t\\tapple, banana, pear, \ <BR>
     *                                  cantaloupe, watermelon, \ <BR>
     *                                  kiwi, mango <BR>
     * The key is "fruits" and the associated element is: 
     * "apple, banana, pear, cantaloupe, watermelon, kiwi, mango".
     * Note that a space appears before each \ so that a space will
     * each comma in the final result; the \, line terminator, and leading white
     * space on the continuation line are merely discarded and are not replaced
     * by one or more other characters. 
     * As a third example, the line: 
     *
     * cheeses <BR>
     * specifies that the key is "cheeses" and the associated element is the
     * empty string "".
     *
     * @param inStream the input stream.
     *
     * @endif
     */
    void load(std::istream& inStream);
    
    /*!
     * @if jp
     *
     * @brief ץѥƥꥹȤꤵ줿ȥ꡼¸
     *
     * 侩Ƥޤ󡣥ץѥƥꥹȤ¸ˡȤƤϡ
     * store(ostream out, string header) ᥽åɤλѤ侩롣
     * Υ᥽åɤ Java Properties ȤθߴΤƤ롣
     *
     * @param out ϥȥ꡼
     * @param header ץѥƥꥹȤε 
     *
     * @else
     *
     * @brief Save the property list to the specified stream
     *
     * It is not recommended. To save the property list,
     * the store(ostream out, string header) method is recommended.
     * This method is defined for compatibility of Java Properties.
     *
     * @param out The output stream
     * @param header A description of the property list
     *
     * @endif
     */
    void save(std::ostream& out, const std::string& header);
    
    /*!
     * @if jp
     *
     * @brief ץѥƥꥹȤϥȥ꡼¸
     *
     * Properties ơ֥Υץѥƥꥹ (ǤΥڥ) load
     * ᥽åɤȤä Properties ơ֥˥ɤΤŬڤʥեޥåȤ
     * ϥȥ꡼˽񤭹ࡣ 
     *
     * Properties ơ֥Υץѥƥꥹ (ǤΥڥ) load
     * ᥽åɤȤä Properties ơ֥˥ɤΤŬڤʥեޥåȤ
     * ϥȥ꡼˽񤭹ࡣȥ꡼ϡISO 8859-1 ʸ
     * 󥳡ǥ󥰤Ѥƽ񤭹ޤ롣 
     * Properties ơ֥ (¸ߤ) Υǥեȥơ֥뤫
     * ץѥƥϡΥ᥽åɤˤäƤϽ񤭹ޤʤ 
     *
     * header  null ǤʤϡASCII ʸ #header ʸ
     * ӹԶڤʸǽ˽ϥȥ꡼˽񤭹ޤޤΤᡢ
     * header ϼ̥ȤȤƻȤȤǤ롣 
     *
     * ˡASCII ʸ #ߤ (Date  toString ᥽åɤˤä
     * ߻郎ΤƱ) Writer ˤäԶڤ
     * ʤ륳ȹԤ񤭹ޤ롣 
     *
     * ³ơ Properties ơ֥Τ٤ƤΥȥ꤬ 1 ԤĽ񤭽Ф롣
     * ƥȥΥʸASCII ʸ=Ϣʸ󤬽񤭹ޤ롣
     * ʸγʸϡץ󥹤Ȥ褹ɬפ뤫
     * ɤǧ롣ASCII ʸ \֡ԡϤ줾 \\\\
     * \\t\\n \\r Ȥƽ񤭹ޤ롣\\u0020 꾮ʸ
     * \\u007E 礭ʸϡб 16  xxxx Ȥä \\uxxxx Ȥ
     * 񤭹ޤ롣߶ʸǤ񤭶ʸǤʤԶʸϡ
     *  \ դƽ񤭹ޤ롣ͤʸ #!= : ϡ
     * ɬɤ褦ˡ˥åդƽ񤭹ޤ롣 
     *
     * ȥ꤬񤭹ޤ줿Ȥǡϥȥ꡼बեå夵롣
     * ϥȥ꡼ϤΥ᥽åɤȤⳫޤޤȤʤ롣 
     *
     * @param out ϥȥ꡼
     * @param header ץѥƥꥹȤε 
     *
     * @else
     *
     * @brief Stores property list to the output stream
     *
     * Write this property list (key and element pairs) in this Properties 
     * table to the output stream in a format suitable for loading into a 
     * Properties table using the load method. The stream is written using the 
     * ISO 8859-1 character encoding. 
     * Properties from the defaults table of this Properties table (if any) are
     * not written out by this method. 
     *
     * If the header argument is not null, then an ASCII # character, the
     * comments string, and a line separator are first written to the output
     * stream. Thus, the header can serve as an identifying comment. 
     *
     * Next, a comment line is always written, consisting of an ASCII #
     * character, the current date and time (as if produced by the toString
     * method of Date for the current time), and a line separator as generated
     * by the Writer. 
     *
     * Then every entry in this Properties table is written out, one per line.
     * For each entry the key string is written, then an ASCII =, then the
     * associated element string. Each character of the key and element strings
     * is examined to see whether it should be rendered as an escape sequence.
     * The ASCII characters \, tab, form feed, newline, and carriage return are
     * written as \\\\, \\t, \\f \\n, and \\r, respectively. Characters less than
     * \\u0020 and characters greater than \\u007E are written as \\uxxxx for the
     * appropriate hexadecimal value xxxx. For the key, all space characters are
     * written with a preceding \ character. For the element, leading space
     * characters, but not embedded or trailing space characters, are written
     * with a preceding \ character. The key and element characters #, !, =, and
     * : are written with a preceding backslash to ensure that they are properly
     * loaded. 
     *
     * After the entries have been written, the output stream is flushed. The
     * output stream remains open after this method returns. 
     *
     * @param out An output stream.
     * @param header The description of the property list.
     *
     * @endif
     */
    void store(std::ostream& out, const std::string& header);
    
    //============================================================
    // other util functions
    //============================================================
    /*!
     * @if jp
     *
     * @brief ץѥƥΥΥꥹȤ vector ֤
     *
     * ᥤץѥƥꥹȤƱ̾ΥĤʤϡǥեȤ
     * ץѥƥꥹȤˤ̤ΥޤࡢΥץѥƥꥹȤˤ뤹٤
     * ΥΥꥹȤ֤ 
     *
     * @return ץѥƥꥹȤˤ뤹٤ƤΥΥꥹȡ
     *         ǥեȤΥץѥƥꥹȤˤ륭ޤ
     *
     * @else
     *
     * @brief Return an vector of all the keys in this property
     *
     * Returns an enumeration of all the keys in this property list, including
     * distinct keys in the default property list if a key of the same name has
     * not already been found from the main properties list.
     *
     * @return A vector of all the keys in this property list, including the
     *         keys in the default property list.
     *
     * @endif
     */
    std::vector<std::string> propertyNames(void) const;
    
    /*!
     * @if jp
     * @brief ץѥƥο
     *
     * ѤߤΥץѥƥ롣
     *
     * @return ץѥƥ
     *
     * @else
     * @brief Get the number of Properties
     *
     * Get the number of Properties that has already set.
     *
     * @return Number of Properties
     *
     * @endif
     */
    int size(void) const;
    
    /*!
     * @if jp
     * @brief Ρɤ
     *
     * ꤷĥΡɤ롣
     * ¸ߤʤӶʸξ 0 ֤
     *
     * @param key оݥΡɤΥ
     *
     * @return оݥΡ
     *
     * @else
     * @brief Get node of properties
     *
     * Get node with specified key.
     *
     * @param key Target node key for getting
     *
     * @return Target node
     *
     * @endif
     */
    Properties* const findNode(const std::string& key) const;
    /*!
     * @if jp
     * @brief Ρɤ
     *
     * ꤷĥΡɤ롣
     * ¸ߤʤӶʸξ 0 ֤
     *
     * @param key оݥΡɤΥ
     *
     * @return оݥΡ
     *
     * @else
     * @brief Get node of properties
     *
     * Get node with specified key.
     *
     * @param key Target node key for getting
     *
     * @return Target node
     *
     * @endif
     */
    Properties& getNode(const std::string& key);
    
    /*!
     * @if jp
     * @brief Ρɤ
     *
     * ꤷĿΡɤ롣
     * Ʊ쥭ĥΡɤϿѤߤξˤϥ顼֤
     *
     * @param key ΡɤΥ
     *
     * @return Ρ
     *         ꤷĥΡɤ¸ߤˤfalse
     *
     * @else
     * @brief Create newly node of Properties
     *
     * Create nowly node with specified key.
     * If the node with the same key has been registered, error will be returned.
     *
     * @param key Newly node key
     *
     * @return Newly node creation result.
     *         false will be returned if the node with specified key has already
     *         existed.
     *
     * @endif
     */
    bool createNode(const std::string& key);
    
    /*!
     * @if jp
     * @brief Ρɤ
     *
     * ꤷ̾Τĥץѥƥ롣
     * ץѥƥ֤
     *
     * @param leaf_name оݥץѥƥ̾
     *
     * @return ץѥƥ
     *
     * @else
     * @brief Remove node of Properties
     *
     * Remove properties with specified name.
     * Properties that were deleted will be returned.
     *
     * @param leaf_name Target property's name for the delete
     *
     * @return Deleted properties
     *
     * @endif
     */
    Properties* removeNode(const char* leaf_name);
    
    /*!
     * @if jp
     * @brief ҥΡɤkey뤫ɤ
     *
     * ꤷĻҥΡɤ¸ߤ뤫ɤǧ롣
     * ¸ߤ硢ҥΡɤ֤
     *
     * @param key ǧоݤΥ
     *
     * @return ҥΡ
     *
     * @else
     * @brief Check whether key exists in the children
     *
     * Check whether the children with specified key exist.
     * If the children exist, they will be returned.
     *
     * @param key Check key
     *
     * @return Children node
     *
     * @endif
     */
    Properties* hasKey(const char* key) const;
    
    /*!
     * @if jp
     * @brief ҥΡɤƺ
     * @else
     * @brief Clear the children
     * @endif
     */
    void clear(void);
    
    /*!
     * @if jp
     * @brief Propertyޡ
     *
     * ߤΥץѥƥꤷץѥƥޡ롣
     *
     * @param prop ޡץѥƥ
     *
     * @return ץѥƥޡ
     *
     * @else
     * @brief Merge properties
     *
     * Merge properties that have set to the current properties.
     *
     * @param prop Properties for the merge
     *
     * @return Property merge result
     *
     * @endif
     */
    Properties& operator<<(const Properties& prop);
    
  protected:
    /*!
     * @if jp
     * @brief ʸ򥭡ͤΥڥʬ䤹
     *
     * Ϳ줿ʸꤵ줿ǥߥǥͤΥڥʬ䤹롣
     * ޤǽͿ줿ʸ':'⤷'='ޤޤ뤫򸡺
     * ɤ餫ʸޤޤƤˤϤǥߥȤƻѤ롣
     * ξȤޤޤƤʤˤϡ' '(ڡ)Ѥʬߤ롣
     * ƤΥǥߥ䤬ޤޤƤʤˤϡͿ줿ʸ򥭡Ȥ
     * ꤷͤ˶ʸꤹ롣
     * ɤΥǥߥˤĤƤ⥨פƤ(ľ'\'ꤵƤ)
     * ˤϡǥߥȤƻѤʤ
     *
     * @param str ʬоʸ
     * @param key ʬ̥
     * @param value ʬ
     *
     * @else
     * @brief Split the string into a pair of the key and the value.
     *
     * Split the given string into a pair of the key and the value with
     * the set delimiter.
     * First, search whether the fist given string includes ':' or '=', and 
     * if either character is included, it is used as delimiter.
     * If neither is included, try to divide it with ' '(space).
     * When all delimiter candidates are not included, set the given string
     * as key then set empty string to the its value.
     * If any delimiter candidate is escaped ('\' is set right before it),
     * this method does not use as delimiter.
     *
     * @param str Target string for the division
     * @param key Division result key
     * @param value Division result value
     *
     * @endif
     */
    static void splitKeyValue(const std::string& str, std::string& key,
			      std::string& value);
    
    /*!
     * @if jp
     * @brief ʸʬ䤹
     *
     * Ϳ줿ʸͿ줿ǥߥʬ䤹롣
     * Ϳ줿ʸ󤬶ξϡ顼֤
     * Ϳ줿ǥߥפƤ(ľ'\'ꤵƤ)
     * ˤϡǥߥȤƻѤʤ
     *
     * @param str ʬоʸ
     * @param delim ǥߥ
     * @param value ʬͥꥹ
     *
     * @return ʬ
     *
     * @else
     * @brief Split the string
     *
     * Divide given string with given delimiter.
     * If the given string is empty, error will be returned.
     * When the given delimiter is escaped ('\' is set right before it)
     * this method does not use as delimiter.
     *
     * @param str Target string for the division
     * @param delim Delimiter
     * @param value Division result list
     *
     * @return Division result
     *
     * @endif
     */
    static bool split(const std::string& str, const char delim,
		      std::vector<std::string>& value);
    
    /*!
     * @if jp
     * @brief ץѥƥ
     *
     * ꥹȤǻꤵ줿ץѥƥ롣
     * ꥹȤǤϡꤹ륭ΥץѥƥǤγشطꥹȷɽ
     * 롣
     * ꤷꥹȤ˳ץѥƥ¸ߤʤNULL֤
     *
     * @param keys оݥץѥƥΥΥꥹɽ
     * @param index ꥹȤγؿ
     * @param curr оݥץѥƥ
     *
     * @return оݥץѥƥ
     *
     * @else
     * @brief Get properties
     *
     * Get properties specified by key list.
     * In the key list, hierarchical relation in the properties is represented
     * by a list format.
     * If properties corresponding to specified key list do not exist,
     * null will be returned.
     *
     * @param keys Target properties's key list representation for getting
     * @param index Number of hierarchy of key list
     * @param curr Target properties for the search
     *
     * @return Target properties for the search
     *
     * @endif
     */
    static Properties* _getNode(std::vector<std::string>& keys,
				std::vector<Properties*>::size_type index,
				const Properties* curr);
    
    /*!
     * @if jp
     * @brief ץѥƥ̾ΥꥹȤ
     *
     * ץѥƥ̾Τ'.'ڤɽꥹȤ롣
     *
     * @param names ץѥƥ̾Υꥹ
     * @param curr_name ߤΥץѥƥ̾
     * @param curr оݥץѥƥ
     *
     * @else
     * @brief Get property name list
     *
     * Get a list expressed by separating each property name with '.'.
     *
     * @param names Name list of property
     * @param curr_name Current property's name
     * @param curr Target properties
     *
     * @endif
     */
    static void _propertiyNames(std::vector<std::string>& names,
				std::string curr_name,
				const Properties* curr);
    
    /*!
     * @if jp
     * @brief ץѥƥ̾ΥꥹȤ¸
     *
     * ץѥƥ̾Τ'.'ڤɽꥹȤ¸롣
     *
     * @param out ץѥƥ̾Υꥹ¸νϥȥ꡼
     * @param curr_name ߤΥץѥƥ̾
     * @param curr оݥץѥƥ
     *
     * @else
     * @brief Store the property name list
     *
     * Store a list expressed by separating each property name with '.'.
     *
     * @param out Output stream of property's name list of save destination.
     * @param curr_name Current property's name
     * @param curr Target properties
     *
     * @endif
     */
    static void _store(std::ostream& out, std::string curr_name,
		       Properties* curr);
    
    /*!
     * @if jp
     * @brief ץѥƥƤ¸
     *
     * ץѥƥꤵ줿Ƥ¸롣
     * ¸ˤϥץѥƥؤοɽղä롣
     * ͤꤵƤʤץѥƥˤĤƤϡǥեͤϤ롣
     *
     * @param out ץѥƥ¸νϥȥ꡼
     * @param curr оݥץѥƥ
     * @param index ߤΥץѥƥ
     *
     * @else
     * @brief Save property's contents
     *
     * Save the contents that were set to the property.
     * The figure represented the depth of the property hierarchy is
     * added when saving.
     * If property with the value that is not set, the default value will
     * be output.
     *
     * @param out Output stream of property's contents of save destination.
     * @param curr Target property
     * @param index Current property hierarchy
     *
     * @endif
     */
    static std::ostream& _dump(std::ostream& out, const Properties& curr,
			       int index);
    
    /*!
     * @if jp
     * @brief ǥȤ
     *
     * ꤵ줿˽äǥȤ֤
     * ֤륤ǥȤϡ2Ĥζ
     *
     * @param index ǥȿλ
     *
     * @return 줿ǥ
     *
     * @else
     * @brief Create indents
     *
     * Return indents according to specified figure.
     * Returned indents are specified figure x two blanks.
     *
     * @param index The specification of Number of indent
     *
     * @return Created indent
     *
     * @endif
     */
    static std::string indent(int index);
    
  private:
    std::string name;
    std::string value;
    std::string default_value;
    Properties* root;
    std::vector<Properties*> leaf;
    const std::string m_empty;

    /*!
     * @if jp
     * @brief Property򥹥ȥ꡼˽Ϥ
     *
     * Property򥹥ȥ꡼˽Ϥ롣
     *
     * @param lhs ϥȥ꡼
     * @param rhs ץѥƥ
     *
     * @return ϥȥ꡼
     *
     * @else
     * @brief Output Properties to stream
     *
     * Output Properties to stream.
     *
     * @param lhs Output stream
     * @param rhs Properties
     *
     * @return Output stream
     *
     * @endif
     */
    friend std::ostream& operator<<(std::ostream& lhs, const Properties& rhs);

  };   // class Properties
};     // namespace coil  
#endif // COIL_PROPERTIES_H

